@@ -20,7 +20,7 @@
 # IN THE SOFTWARE.
 
 __author__ = 'Niklas Rosenstein <rosensteinniklas@gmail.com>'
-__version__ = '0.1.0'
+__version__ = '0.2.1'
 __all__ = [
   'Location',
   'Decoration',
@@ -40,9 +40,9 @@ __all__ = [
 ]
 
 
-from nr.databind.core import Field, ObjectMapper, ProxyType, Struct, UnionType
-from nr.databind.json import JsonModule
-from typing import Any, Callable, Dict, Iterable, List, Optional, TextIO, Union
+from nr.databind.core import Field, ObjectMapper, ProxyType, Struct, UnionType, decorations  # type: ignore
+from nr.databind.json import JsonModule  # type: ignore
+from typing import Any, Callable, Dict, Iterable, List, Optional, TextIO, Union, cast
 import enum
 import io
 import json
@@ -51,16 +51,19 @@ _ClassProxy = ProxyType()
 _mapper = ObjectMapper(JsonModule())
 
 
+@decorations.SkipDefaults()
 class Location(Struct):
-  filename = Field(str, nullable=True)
+  filename = Field(str, default=None)
   lineno = Field(int)
 
 
+@decorations.SkipDefaults()
 class Decoration(Struct):
   name = Field(str)
-  args = Field(str, nullable=True)
+  args = Field(str, default=None)
 
 
+@decorations.SkipDefaults()
 class Argument(Struct):
   class Type(enum.Enum):
     PositionalOnly = 0
@@ -70,34 +73,36 @@ class Argument(Struct):
     KeywordRemainder = 4
   name = Field(str)
   type = Field(Type)
-  decorations = Field([Decoration], nullable=True)
-  datatype = Field(str, nullable=True)
-  default_value = Field(str, nullable=True)
+  decorations = Field([Decoration], default=None)
+  datatype = Field(str, default=None)
+  default_value = Field(str, default=None)
 
 
+@decorations.SkipDefaults()
+@decorations.KeywordsOnlyConstructor()
 class ApiObject(Struct):
   name = Field(str, prominent=True)
-  location = Field(Location, nullable=True)
-  docstring = Field(str, nullable=True)
+  location = Field(Location, default=None)
+  docstring = Field(str, default=None)
 
 
 class Data(ApiObject):
-  datatype = Field(str, nullable=True)
-  value = Field(str, nullable=True)
+  datatype = Field(str, default=None)
+  value = Field(str, default=None)
 
 
 class Function(ApiObject):
-  modifiers = Field([str], nullable=True)
+  modifiers = Field([str], default=None)
   args = Field([Argument])
-  return_type = Field(str, nullable=True)
-  decorations = Field([Decoration], nullable=True)
+  return_type = Field(str, default=None)
+  decorations = Field([Decoration], default=None)
 
 
 @_ClassProxy.implementation
 class Class(ApiObject):
-  metaclass = Field(str, nullable=True)
-  bases = Field([str], nullable=True)
-  decorations = Field([Decoration], nullable=True)
+  metaclass = Field(str, default=None)
+  bases = Field([str], default=None)
+  decorations = Field([Decoration], default=None)
   members = Field([UnionType({
     'data': Data,
     'function': Function,
@@ -132,7 +137,7 @@ def load_module(
   filename = filename or getattr(source, 'name', None)
 
   if isinstance(source, str):
-    with open(source) as fp:
+    with io.open(source, encoding='utf-8') as fp:
       return load_module(fp, source, loader)
   elif hasattr(source, 'read'):
     source = loader(source)
@@ -154,11 +159,11 @@ def load_modules(
   filename = filename or getattr(source, 'name', None)
 
   if isinstance(source, str):
-    with open(source) as fp:
+    with io.open(source, encoding='utf-8') as fp:
       yield from load_modules(fp, source, loader)
     return
   elif hasattr(source, 'read'):
-    source = (loader(io.StringIO(line)) for line in source)
+    source = (loader(io.StringIO(line)) for line in cast(TextIO, source))
 
   for data in source:
     yield _mapper.deserialize(data, Module, filename=filename)
@@ -174,7 +179,7 @@ def dump_module(
   """
 
   if isinstance(target, str):
-    with open(target, 'w') as fp:
+    with io.open(target, 'w', encoding='utf-8') as fp:
       dump_module(module, fp, dumper)
     return None
 
@@ -182,6 +187,7 @@ def dump_module(
   if target:
     dumper(data, target)
     target.write('\n')
+    return None
   else:
     return data
 
@@ -241,7 +247,7 @@ class ReverseMap:
 
   def __init__(self, modules: List[Module]) -> None:
     self._modules = modules
-    self._reverse_map = {}
+    self._reverse_map: Dict[int, Optional[ApiObject]] = {}
     for module in modules:
       self._init(module, None)
 
@@ -258,9 +264,10 @@ class ReverseMap:
 
   def path(self, obj: ApiObject) -> List[ApiObject]:
     result = []
-    while obj:
-      result.append(obj)
-      obj = self.get_parent(obj)
+    current: Optional[ApiObject] = obj
+    while current:
+      result.append(current)
+      current = self.get_parent(current)
     result.reverse()
     return result
 
